"""Use signal.alarm to trigger a timeout after some seconds."""

import collections
import contextlib
import signal
import types


class AlarmError(Exception):
    """Exception thrown on timeout."""


def _alarm_handler(_signum: int, _frame: types.FrameType | None) -> None:
    """Raise an AlarmError."""
    raise AlarmError()


@contextlib.contextmanager
def timeout(timeout_s: int) -> collections.abc.Iterator[None]:
    """Use signal.alarm to trigger a timeout after some seconds.

    An AlarmError will be raised if the timeout is hit.
    """
    signal.signal(signal.SIGALRM, _alarm_handler)
    signal.alarm(timeout_s)
    try:
        yield
    finally:
        signal.alarm(0)
        signal.signal(signal.SIGALRM, signal.SIG_DFL)
